#!/usr/bin/perl -w

# Script for computing Kendall's Tau 
#   (adapted from a script by Christof Monz)
#
# Copyright (C) 2007, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# usage: ./compute-kendall-tau.pl [file1] [file2]
#
# takes two files of the form:
# name1 score1
# name2 score2
# ...

$USAGE = "usage: $0 [file1] [file2]\n\nfiles of the form:\ntag1 score1\ntag2 score2\n...\n";

$file1=$ARGV[0] or die $USAGE;
$file2=$ARGV[1] or die $USAGE;

open(FILE1,"<$file1") || die("can't open file $file1:$!");
while(defined($line=<FILE1>)) {
    next if($line=~/^[\s\t]*\n/);
    chomp($line);
    ($id,$val) = split(/[\s\t]/,$line);
    $id=~tr/A-Z/a-z/;
    $hasha{$id} = $val;
};
close(FILE1);

open(FILE2,"<$file2") || die("can't open file $file1:$!");
while(defined($line=<FILE2>)) {
    next if($line=~/^[\s\t]*\n/);
    chomp($line);
    ($id,$val) = split(/[\s\t]/,$line);
    $id=~tr/A-Z/a-z/;
    $hashb{$id} = $val;
};
close(FILE2);

$k_tau = kendall_tau(\%hasha,\%hashb);
print "tau: $k_tau\n";

sub kendall_tau {
    my($i,$j);
    my($concordants,$discordants,$list_length,$tau);
    my(@elements1,%pairs,$extra1,$extra2);

    ($hash_1,$hash_2) = @_;


    $concordants = 0;
    $discordants = 0;
    $extra1=0;
    $extra2=0;

    foreach $el1 (keys %$hash_1) {
	if(defined($hash_2->{$el1})) {
	    push(@elements1,$el1);
	};
    };


    for($i=0; $i<@elements1-1; $i++) {
	for($j=$i+1; $j<@elements1; $j++) {
	    $pairs{$elements1[$i]}{$elements1[$j]} = 1;
	};
    };

    foreach $el1 (keys %pairs) {
	foreach $el2 (keys %{ $pairs{$el1} }) {
	    if($hash_1->{$el1} > $hash_1->{$el2}
	       && $hash_2->{$el1} > $hash_2->{$el2}) {
		$concordants++;
	    } elsif($hash_1->{$el1} < $hash_1->{$el2}
		    && $hash_2->{$el1} < $hash_2->{$el2}) {
		$concordants++;
	    } elsif($hash_1->{$el1} == $hash_1->{$el2}
		    && $hash_2->{$el1} == $hash_2->{$el2}) {
	    } elsif($hash_1->{$el1} == $hash_1->{$el2}) {
		$extra2++;
	    } elsif($hash_2->{$el1} == $hash_2->{$el2}) {
		$extra1++;
	    } else {
		$discordants++;
	    };
	};
    };


    if(($concordants + $discordants + $extra2) == 0
       || ($concordants + $discordants + $extra1) == 0) {
	return 0;
    };

    $tau = ($concordants - $discordants)
	   /(sqrt($concordants + $discordants + $extra2) 
	     * sqrt($concordants + $discordants + $extra1));
 

    return $tau;

};
